//importing functions and objects from D3 library
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  curveBasis
} from 'd3';

// svg element and storing values in width and height variables
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');


const render = data => {
  const title = 'Female Suicides of Age 25-34 years'; // title 
  const xValue = d => d.year; //extracting exact data for X-axis
  const xAxisLabel = 'Year'; //labelling X-axis
  const yValue = d => d.suicides_no; //extracting exact data for Y-axis
  const circleRadius = 4;
  const yAxisLabel = 'Number of Suicides'; //labelling X-axis
  //styles of plot
  const margin = { top: 60, right: 40, bottom: 88, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  //map the data values to the x and y coordinates on the chart
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0,innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
 
  //generate the x-axis and y-axis with appropriate tick marks and labels.
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(15);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  // defining attributes and styles for y-axis label
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -70) // y-coordinate for Y-label positioning
      .attr('x', -innerHeight / 2) // x-coordinate for Y-label positioning
      .attr('fill', 'red') // label is in red color
      .attr('transform', `rotate(-90)`) // rotating the text 
      .attr('text-anchor', 'middle') // text aligned in middle
      .style("font-size","1.5em") // specifying text size
      .style("font-style","italic") // specifying text style
      .style("font-weight","bold") // making text bold
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  // defining attributes and styles for x-axis label
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 50) // y-coordinate for Y-label positioning
      .attr('x', innerWidth / 2) // x-coordinate for Y-label positioning
      .attr('fill', 'red')  // label is in red color
      .style("font-size","1.5em") // specifying text size
      .style("font-style","italic") // specifying text style
      .style("font-weight","bold") // making text bold
      .text(xAxisLabel);
  
  //to generate the line path for the line chart based on the data.
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)))
 	  .curve(curveBasis);
 //the SVG elements for the x-axis, y-axis, line path, and title are added to the chart. 
  g.append('path')
      .attr('class', 'line-path')
      .attr('d', lineGenerator(data));
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -25)
      .attr('x', 70)
      .text(title);
};
//gist URL of my dataset
//load the data from a CSV file 
csv('https://gist.githubusercontent.com/yamunayennem/2b33182daebe91db20a7b1ec174f85d0/raw/9510498318bc5ef54448dfdff53bd8f7b2345aa9/deaths_female.csv')
  .then(data => {
    data.forEach(d => {
      d.suicides_no = +d.suicides_no;
      d.year = +d.year;
    });
    render(data); // render the line chart by calling the render function with the loaded data
  });